@@ -5,6 +5,7 @@
 #include <ctime>
 #include "matrix4.h"
 #include "loubar.h"
+#include "m4_kinship_type.h"
 
 using namespace Rcpp;
 using namespace RcppParallel;
@@ -18,18 +19,18 @@ struct paraKin_p : public Worker {
   const std::vector<double> p;
   const size_t ncol;
   const size_t true_ncol;
-  const double n;
+  const Ktype n;
   // output
-  double * K;
+  Ktype * K __attribute__ ((aligned(16)));
   
   // constructeurs
   paraKin_p(matrix4 & A, std::vector<double> p) : A(A), p(p), n(A.nrow-1),  // ******* BEWARE THE nrow-1 !!! ***********
         ncol(A.ncol), true_ncol(A.true_ncol) { 
-          K = new double[(ncol*(ncol+1))/2];
+          K = new Ktype[(ncol*(ncol+1))/2];
           std::fill(K, K+(ncol*(ncol+1))/2, 0);
         }
   paraKin_p(paraKin_p & Q, Split) : A(Q.A), p(Q.p), n(Q.n), ncol(Q.ncol), true_ncol(Q.true_ncol) {
-          K = new double[(ncol*(ncol+1))/2];
+          K = new Ktype[(ncol*(ncol+1))/2];
           std::fill(K, K+(ncol*(ncol+1))/2, 0);
         }
 
@@ -40,15 +41,15 @@ struct paraKin_p : public Worker {
 
   // worker !
   void operator()(size_t beg, size_t end) {
-    double gg[16];
+    Ktype gg[16] __attribute__ ((aligned(16)));
     gg[3] = gg[7] = gg[11] = gg[12] = gg[13] = gg[14] = gg[15] = 0;
     for(size_t i = beg; i < end; i++) {
-      double p_ = p[i]; 
+      Ktype p_ = (Ktype) p[i]; 
       if(p_ == 0 || p_ == 1) continue;
-      double w_ = 1/sqrt(2*p_*(1-p_)*n);
-      double v0 = -2*p_*w_;
-      double v1 = (1-2*p_)*w_;
-      double v2 = (2-2*p_)*w_;
+      Ktype w_ = 1/sqrt(2*p_*(1-p_)*n);
+      Ktype v0 = -2*p_*w_;
+      Ktype v1 = (1-2*p_)*w_;
+      Ktype v2 = (2-2*p_)*w_;
       size_t k = 0;
 
       gg[0] = v0*v0; 
@@ -60,18 +61,18 @@ struct paraKin_p : public Worker {
       char * dd = A.data[i];
       for(size_t j1 = 0; j1 < true_ncol; j1++) {
         char x1 = dd[j1];
-        for(int ss1 = 0; ss1 < 4 && 4*j1 + ss1 < ncol; ss1++) {
-          double * ggg = gg + ((x1&3)<<2);
+        for(unsigned int ss1 = 0; (ss1 < 4) && (4*j1 + ss1 < ncol); ss1++) {
+          Ktype * ggg __attribute__ ((aligned(16))) = gg + ((x1&3)<<2);
           for(size_t j2 = 0; j2 < j1; j2++) {
             char x2 = dd[j2];
-            for(int ss2 = 0; ss2 < 4; ss2++) {
+            for(unsigned int ss2 = 0; ss2 < 4; ss2++) {
               K[k++] += ggg[x2&3];
               x2 >>= 2;
             }
           }
           size_t j2 = j1;
           char x2 = dd[j2];
-          for(int ss2 = 0; ss2 <= ss1; ss2++) {
+          for(unsigned int ss2 = 0; ss2 <= ss1; ss2++) {
             K[k++] += ggg[x2&3];
             x2 >>= 2;
           }
@@ -83,7 +84,7 @@ struct paraKin_p : public Worker {
 
   // recoller
   void join(const paraKin_p & Q) {
-    std::transform(K, K + (ncol*(ncol+1))/2, Q.K, K, std::plus<double>());
+    std::transform(K, K + (ncol*(ncol+1))/2, Q.K, K, std::plus<Ktype>());
     // autrement dit : K += Q.K;
   }
 
@@ -101,7 +102,7 @@ NumericMatrix Kinship_p(XPtr<matrix4> p_A, const std::vector<double> & p, int ch
   size_t k = 0;
   for(size_t i = 0; i < p_A->ncol; i++) {
     for(size_t j = 0; j <= i; j++) {
-      Y(j,i) = X.K[k++];
+      Y(j,i) = (double) X.K[k++];
     }
   }
 
@@ -109,7 +110,7 @@ NumericMatrix Kinship_p(XPtr<matrix4> p_A, const std::vector<double> & p, int ch
   k = 0;
   for(size_t i = 0; i < p_A->ncol; i++) {
     for(size_t j = 0; j <= i; j++) {
-      Y(i,j) = X.K[k++]; // ou Y(j,i)
+      Y(i,j) = (double) X.K[k++]; // ou Y(j,i)
     }
   }
   
